package com.Felipe.Drawing;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

public class GraphPanel extends JPanel {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	//zde se uchovvaj hodnoty, kter ns zajmaj
	class Graf{
		double []graf; 	//aktualn poloha
		double maxY, minY;  //maximum a minimum
		int delX = 0;		//posun na ose x
	}
	
	private List<Graf> grafs = new ArrayList<Graf>(); //list of all grafs
	
	//Hodnoty, kter ovliuj vykreslovn(renderovan) grafu
	int PAD = 15;		//padding
	double maxY, minY;	//minimum a maximum pro vechny grafy
	int maxX;			//max dlka kadho grafu
	
	//Colors to use
	Color colors[]=new Color[]{Color.BLUE,Color.RED, Color.GREEN, Color.ORANGE,  Color.MAGENTA, Color.YELLOW, Color.DARK_GRAY};
	
	//---Vbrov parametry----//
	private int selectedGraph = -1;
	private int selectedX = -1; 
	
	//---------------------------------Udlost---------------------------//
	
	//udlost
	public interface Runnable<T1, T2>{
		void run(T1 p1, T2 p2);
	}

	Runnable<Integer, Integer> showPointChanged; //nastane pokud se budete pohybovat nad grafem
	
	//----------------------------------------------------------------//
	
public GraphPanel(){
	
	maxY=Integer.MIN_VALUE;
	minY=Integer.MAX_VALUE;
	maxX = 0;
	setBorder(BorderFactory.createLineBorder(Color.black));
	
	//---pokud se my pohybuje nad grafem tak---
	this.addMouseMotionListener(new MouseMotionAdapter() {
		@Override
		public void mouseMoved(MouseEvent e) {
			if(maxX==0) return; //tzn. dn graf
			
			
			double x, y, w;
			w = getWidth();

			//dopotn stupnice
			double scaleY = (double)(getHeight() - 2*PAD)/(maxY-minY);
			double scaleX = (double)(w - 2*PAD)/(maxX);
			
			x = e.getX();
			y = ((getHeight()- e.getY())/scaleY + minY);
			
			setSelectedGraph(-1);
			setSelectedX(-1);

			/*
			 * najdi graf odpovdajc pozici myi 
			 */
			
			double minDiff = Double.MAX_VALUE, diff;
			int i;
			
			for(int g = 0; g<grafs.size();g++){ //znovu pouij u vech graf
				
				Graf gr = grafs.get(g);
				
				i = (int) ((x-PAD-gr.delX)/scaleX); //vypotej index zaloen na "x" a "delX" [obracen k postupu p vpotu plotu "i"]
				
				if(grafs.get(g).graf.length>i&&i>=0){
					
					diff = Math.abs(y-gr.graf[i]);
					
					if(minDiff> diff){
						minDiff = diff;
						setSelectedGraph(g);
						setSelectedX(i);
					}
					
				}
			}
			
			//Spout udlosti
			if(showPointChanged!=null) showPointChanged.run(getSelectedGraph(), getSelectedX());
		}
	});
	
	
}
public Dimension getPreferredSize() {
    return new Dimension(250,200);
}


public void addGraph(double []data){
	if(data==null) return;
	Graf g = new Graf();
	
	g.maxY = Double.MIN_VALUE;
	g.minY = Double.MAX_VALUE;
	
	setSelectedGraph(-1); setSelectedX(-1);
	
	//iteracu u vech graf tak aby byli nalezena min a max
	for(double b:data){
		g.maxY = Math.max(g.maxY, b);
		g.minY = Math.min(g.minY, b);
	}
	
	//aktualizace globalnich min a max hodnot
	maxX = Math.max(maxX, data.length);
	maxY = Math.max(g.maxY, maxY);
	minY = Math.min(g.minY, minY);
	
	g.graf = data;
	grafs.add(g);
	
	repaint(); //voln metody znovu pekreslen grafu
}

public int getNumGraphs(){
	return grafs.size();
}

public void clearGraphs(){
	grafs.clear();
	
	maxY=Integer.MIN_VALUE;
	minY=Integer.MAX_VALUE;
	maxX = 0;
	repaint();
}

//zde je ji vykreslovn u konce
public void paintComponent(Graphics g) {
    super.paintComponent(g);       

    if(maxX==0) return;
    
    Graphics2D g2 = (Graphics2D)g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    
    int w = getWidth();
    int h = getHeight();
    
    //potn stupnice
    double scaleX = (double)(w - 2*PAD)/(maxX);
    double scaleY = (double)(h - 2*PAD)/(maxY-minY);
    
    //renderovan vech graf
    int col = 0;
    for(Graf graf:grafs){
    	g2.setColor(colors[(col++)% colors.length]);
    	for(int i = 1; i < graf.graf.length; i++) {

        	int x2 = (int) (PAD + i*scaleX)+graf.delX;
        	int y2 = (int) (h - PAD - scaleY*(graf.graf[i]-minY));
        	int x1 = (int) (PAD + (i-1)*scaleX)+graf.delX;
        	int y1 = (int) (h - PAD - scaleY*(graf.graf[i-1]-minY));

        	g2.drawLine(x1, y1, x2, y2);
        }
        

    }
    
    //renderovan textu
    g2.setPaint(Color.black);
    
    //pokud je vybran njak bod vykresli jej
    if(getSelectedGraph()!=-1&&getSelectedX()!=-1)
    {
    	Graf gr = grafs.get(getSelectedGraph());
    	if(gr.graf.length>getSelectedX()){
    		int x, y;
    		x = (int) (PAD + getSelectedX()*scaleX)+gr.delX;
    		y = (int) (h - PAD - scaleY*(gr.graf[getSelectedX()]-minY));

    		g2.drawString(gr.graf[getSelectedX()]+"", w-100, 15);	//zobraz hodnoty
    		g2.drawOval(x-2, y-2, 4, 4);							//vykreslen kruhu
    	}
    }
    
    //zobraz minima mamxima
    g2.drawString((long)maxY+"", PAD/2, (int) (PAD));
    g2.drawString((long)minY+"", PAD/2, (int) (h-PAD));
    
    
}  

public void setShowPoint(int sGraph, int sX){
	setSelectedGraph(sGraph);
	setSelectedX(sX);
	GraphPanel.this.repaint();
}
public void setShowPoint(int sGraph, double prg){
	double x, w;
	w = getWidth();

	//potn stupnice
	double scaleX = (double)(w - 2*PAD)/(maxX);
	
	x = prg*w;
	
	int i = (int) ((x-PAD-grafs.get(0).delX)/scaleX); //vypotej index zaloen na "x" a "delX" [obracen k postupu p vpotu plotu "i"]
	
	setSelectedGraph(sGraph);
	setSelectedX(i);
	GraphPanel.this.repaint();
}

public void setShowPointChanged(Runnable<Integer, Integer> r){
	showPointChanged = r;
	
}
public int getSelectedGraph() {
	return selectedGraph;
}
public void setSelectedGraph(int selectedGraph) {
	this.selectedGraph = selectedGraph;
}
public int getSelectedX() {
	return selectedX;
}
public void setSelectedX(int selectedX) {
	if(selectedX<0) selectedX=-1;
	this.selectedX = selectedX;
}
public double getSelectedValue(){
	return grafs.get(getSelectedGraph()).graf[getSelectedX()];
}
public void addDeltaX(int g, int del){
	grafs.get(g).delX+=del;
	repaint();
}
}
