package com.Felipe.Drawing;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import com.Felipe.Wave.RezOtiskuZvuku;
import com.Felipe.Wave.DetektorAnylyzaZvuku;

public class WaveMatchViewer extends JPanel{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private DetektorAnylyzaZvuku.FingerprintMatches fpm;
	int totalFrames;
	
	public WaveMatchViewer(){
		setBorder(BorderFactory.createLineBorder(Color.black));
		//this.setBackground(Color.black);
		
		this.addMouseMotionListener(new MouseMotionAdapter() {
			@Override
			public void mouseMoved(MouseEvent e) {
				if(getFpm()==null) return;
				int w = getWidth();
			    double scaleX = (double)w/totalFrames;
			    
			    WaveMatchViewer.this.setToolTipText("");
			    
			    int  fn = (int) (e.getX()/scaleX);
			    for(RezOtiskuZvuku fp:getFpm().matches){
			    	if(fp.frameNumber>fn) break;
			    	if(fp.frameNumber==fn){
			    		WaveMatchViewer.this.setToolTipText("<html>"+ fp.getString().replaceAll("\r\n", "<br>") + "</html>");
			    		break;
			    	}
			    }
			    
			}
		});
	}
	
	public void setTotalFrames(int totalFrames){
		this.totalFrames = totalFrames;
		
	}
	
	public void paintComponent(Graphics g) {
	    super.paintComponent(g);
	    if(getFpm()==null) return;
	    Graphics2D g2 = (Graphics2D)g;
	    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	    
	    int w = getWidth();
	    int h = getHeight();
	    
	    double scaleX = (double)w/totalFrames;
	    int radius = (int) Math.max(1, Math.min(scaleX, h/2));
	    g2.drawString(getFpm().actionName, 0, h/2);
	    g2.setColor(Color.BLUE);
	    for(RezOtiskuZvuku fp:getFpm().matches){
	    	int x = (int) (scaleX*fp.frameNumber);
	    	g2.drawOval(x, h/2, radius, radius);
	    	
	    }
	}

	public DetektorAnylyzaZvuku.FingerprintMatches getFpm() {
		return fpm;
	}

	public void setFpm(DetektorAnylyzaZvuku.FingerprintMatches fpm) {
		this.fpm = fpm;
		repaint();
	}
}
