package com.Felipe;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;

public class PrehravacZvuku implements Runnable {

	SourceDataLine line;

	Thread vlakno;
	
	AudioInputStream audioInputStream;
	final int bufSize = 16384;
	double progress = 0;

	Runnable onShutdown;
	Runnable playbackTimer;
	
	public void setOnShutdown(Runnable r){
		onShutdown=r;
	}
	public void setPlaybackTimer(Runnable r){
		playbackTimer=r;
	}
	public void start(AudioInputStream audioInputStream) {
		this.audioInputStream = audioInputStream;
		vlakno = new Thread(this);
		vlakno.setName("Pehrvn");
		vlakno.start();
	}

	public void stop() {
		vlakno = null;
	}

	public void shutDown(String msg){
		if(onShutdown!=null)
			onShutdown.run();
	}

    public void run() {

      // kontrola zda je opravdu co pehrt
      if (audioInputStream == null) {
        shutDown("Audio k pehrvn nen nateno");
        return;
      }
      // Reset zvukovho streamu
      try {
        audioInputStream.reset();
      } catch (Exception e) {
        shutDown("Error: stream nelze resetovat\n" + e);
        return;
      }

      // nastaven poadovanho formtu pro pehrvn AudioInputStream

      AudioFormat.Encoding encoding = AudioFormat.Encoding.PCM_SIGNED;
      float rate = 44100.0f;
      int channels = 2;
      int sampleSize = 16;
      boolean bigEndian = true;

      AudioFormat format = new AudioFormat(encoding, rate, sampleSize, channels, (sampleSize / 8)
          * channels, rate, bigEndian);

      AudioInputStream playbackInputStream = AudioSystem.getAudioInputStream(format,
          audioInputStream);

      if (playbackInputStream == null) {
        shutDown("Nelze konvertovat stream formatu " + audioInputStream + " do formatu " + format);
        return;
      }

     
      // definoan poadovanch atribut pro na line
      // ujitn o tom zda jsou parametry na line shodn s parametry streamu

      DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);
      if (!AudioSystem.isLineSupported(info)) {
        shutDown("Line porovnvn " + info + " nen podporovno.");
        return;
      }

      //  get a oteven zdrojovch dat pro pehrvn

      try {
        line = (SourceDataLine) AudioSystem.getLine(info);
        line.open(format, bufSize);
      } catch (LineUnavailableException ex) {
        shutDown("Line nelze otevt: " + ex);
        return;
      }

      // pehrvn nahrtho audio zznamu

      int frameSizeInBytes = format.getFrameSize();
      int bufferLengthInFrames = line.getBufferSize() / 8;
      int bufferLengthInBytes = bufferLengthInFrames * frameSizeInBytes;
      byte[] data = new byte[bufferLengthInBytes];
      int numBytesRead = 0;
      double ctr=0;

      // (start line) ze zdrojovho souboru
      line.start();

      while (vlakno != null) {
        try {
          if ((numBytesRead = playbackInputStream.read(data)) == -1) {
            break;
          }
          ctr+=numBytesRead;
          progress=(ctr/(playbackInputStream.getFrameLength()*frameSizeInBytes));
          int numBytesRemaining = numBytesRead;
          while (numBytesRemaining > 0) {
            numBytesRemaining -= line.write(data, 0, numBytesRemaining);
          }
          if(playbackTimer!=null) playbackTimer.run();
        } catch (Exception e) {
          shutDown("Error v prbhu pehrvn: " + e);
          break;
        }
      }
      // Poadavky
      // aplikace se dostala na konec streamu
      // stopni a ukoni line.
      if (vlakno != null) {
        line.drain();
      }
      line.stop();
      line.close();
      line = null;
      shutDown(null);
    }
  } // konec tdy Playback//Pehrvn
