package com.Felipe;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.sound.sampled.AudioFileFormat.Type;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.TargetDataLine;

/*
 * Tda pro ely zznamu zvuku
 */
public class ZznamZvuku implements Runnable {

    TargetDataLine line;

    Thread thread;			     //vlkno pro zznam zvuku
    ByteArrayInputStream is;
    AudioFormat format;
    AudioInputStream ais;
    
    Runnable recordingCompleted; //event raised after recording is complete
    boolean debug;
    
    public ZznamZvuku(boolean debug){
    	 is = null;
    	 this.debug = debug;	//debug = true ulo nahrty zvuk do tmp.wave

        // zde je nutn definovat poadavky pro atributy ely line
        // zajistit kompatibilitu line a zvuku

        AudioFormat.Encoding encoding = AudioFormat.Encoding.PCM_SIGNED;
        float rate = 44100.0f;
        int channels = 1;
        int sampleSize = 16;        //musicg zajiuje podporu pouze pro 8 nebo 16
        boolean bigEndian = false;

        format = new AudioFormat(encoding, rate, sampleSize, channels, (sampleSize / 8)
            * channels, rate, bigEndian);

    }
    
    //Zatek nahrvn zvuku
    public void start() {
      thread = new Thread(this);
      thread.setName("Zaznam");
      thread.start();
    }

    //okamit zastav nahrvn
    public void stop() {
      thread = null; //perueni/break vlkna
    }

    private void shutDown(String message) {
      
    }

    public void run() {

      
      DataLine.Info info = new DataLine.Info(TargetDataLine.class, format);

      if (!AudioSystem.isLineSupported(info)) {
        shutDown("Line porovnvn " + info + " nen podporovno.");
        return;
      }

      // zvolen a oteven clov line

      try {
        line = (TargetDataLine) AudioSystem.getLine(info);
        line.open(format, line.getBufferSize());
      } catch (LineUnavailableException ex) {
        shutDown("Line nelze otevt: " + ex);
        return;
      } catch (SecurityException ex) {
        shutDown(ex.toString());
        //JavaSound.showInfoDialog();
        return;
      } catch (Exception ex) {
        shutDown(ex.toString());
        return;
      }

      // pehrt zvukovho zznamu
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      int frameSizeInBytes = format.getFrameSize();
      int bufferLengthInFrames = line.getBufferSize() / 8;
      int bufferLengthInBytes = bufferLengthInFrames * frameSizeInBytes;
      byte[] data = new byte[bufferLengthInBytes];
      int numBytesRead;

      line.start();

      while (thread != null) {
        if ((numBytesRead = line.read(data, 0, bufferLengthInBytes)) == -1) {
          break;
        }
        out.write(data, 0, numBytesRead);
      }

      // doshli jsme konce streamu
      // program zastav a zave line
      line.stop();
      line.close();
      line = null;

      // zastav a zavi output stream
      try {
        out.flush();
        out.close();
      } catch (IOException ex) {
        ex.printStackTrace();
      }

      // nahrvn bajt pes input stream pro pehrvn

      byte audioBytes[] = out.toByteArray();
      is = new ByteArrayInputStream(audioBytes);
      ais = new AudioInputStream(is, format, audioBytes.length / format.getFrameSize());
      try {
		ais.reset();
		ByteArrayOutputStream tmp = new ByteArrayOutputStream();
	    AudioSystem.write(ais, Type.WAVE, tmp);
	    ais.reset();
	    if(debug)
	    	AudioSystem.write(ais, Type.WAVE, new File("tmp.wave"));
	    is = new ByteArrayInputStream(tmp.toByteArray());
	    if(recordingCompleted!=null) recordingCompleted.run();
	      
	} catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
      
      
    }

    public AudioFormat getFormat(){
    	return format;
    }
    public void setOnRecordingCompleted(Runnable r){
    	//nastav udlost
    	recordingCompleted=r;
    }

    public InputStream getRecordedStream() throws IOException{
    	if(is==null) return null;
    	is.reset(); //reset na zatek
    	return is;
    }
    public AudioInputStream getAudioStream() throws IOException{
    	if(ais==null) return null;
    	ais.reset();
    	return ais;
    }
    
}